/***********************************************************************/
/*                                                                     */
/*  FILE        :Main.c                                                */
/*  DATE        :                                                      */
/*  DESCRIPTION :Main Program                                          */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
#include "iodefine.h"

typedef unsigned char byte;
typedef unsigned int  word;


#define LCD_CTRL_PORTMODE PM7
#define LCD_DATA_PORTMODE PM1

#define LCD_OUTPUT_PORT P1
#define LCD_CTRL_PORT   P7

#define DISPLAY_ON   0x3f
#define DISPLAY_OFF  0x3e

#define RS      0x01
#define RW      0x02
#define E       0x04
#define _CS1    0x08
#define _CS2    0x10
#define CS_ALL  0x18

#define SetRS(m) m?(LCD_CTRL_PORT|=RS):(LCD_CTRL_PORT&=~RS)
#define SetRW(m) m?(LCD_CTRL_PORT|=RW):(LCD_CTRL_PORT&=~RW)
#define SetE(m) m?(LCD_CTRL_PORT|=E):(LCD_CTRL_PORT&=~E)
#define SetCS1(m) m?(LCD_CTRL_PORT|=_CS1):(LCD_CTRL_PORT&=~_CS1)
#define SetCS2(m) m?(LCD_CTRL_PORT|=_CS2):(LCD_CTRL_PORT&=~_CS2)

char compare[8][128]= { {0x00,0x00,0x00,0x00, 0x00, 0x00, 0x00, 0x00}, };
int fraction;
byte nGraphic_Global_Cursor_X=0xb8,nGraphic_Global_Cursor_Y=0x40;
int status=0;

void main(void);
void port_Init(void);
void lcd_Init(void);
void write_Cmd(byte cs,byte cmd);
void write_Data(byte cs,byte ch);

void lcd_Clear(void);


void DrawPixel(int x, int y);
void ClearPixel(int x, int y);

void DrawLine(int X0, int Y0, int X1, int Y1);
void ClearLine(int X0, int Y0, int X1, int Y1);

void DrawRect(int left,int top,int right,int bottom,int bFilling);
void ClearRect(int left, int top, int right, int bottom);

void DrawCircle(int x,int y,int radius,int bFilling);
void ClearCircle(int x, int y, int radius);

void delay_ms(int ms);
void delay_us(register int us);

void main(void)
{
  int i;
  port_Init();                         // |[g
  lcd_Init();                          // LCD
  lcd_Clear();                         // LCDNA
  while(1)
  {
    for(i=0;i<14;i++)
    {
      DrawCircle(64,32,i*2,0);         // ~`
      delay_ms(100);                   // 100ms wait
    }
    for(i=0;i<14;i++)
    {
      ClearCircle(64,32,i*2);          // NA~`
      delay_ms(100);                   // 100ms wait
    }
  }

}
void delay_ms(int ms)
{
  volatile int i,j;
  for(i=0;i<ms;i++)
  {
    for(j=0;j<2963;j++)
    {

    }
  }
}
void delay_us(register int us)
{
  us<<=1;
  us-=8;
  while(us)
  {
    us--;
  }
}
void port_Init(void)
{
  LCD_DATA_PORTMODE=0x00;              // f[^|[gSo
  LCD_CTRL_PORTMODE=0x00;              // |[gSo
  LCD_CTRL_PORT=0x00;                  // |[gSLow
  LCD_OUTPUT_PORT=0x00;                // f[^o͑SLow
}
void write_Cmd(byte cs,byte cmd)
{
  LCD_CTRL_PORT = cs & CS_ALL;         // CSo
  SetE(1);                             // E High
  LCD_OUTPUT_PORT = cmd;               // R}ho
  delay_us(10);                        // 10us wait
  SetE(0);                             // E Low
  delay_us(10);                        // 10us wait
  LCD_CTRL_PORT=0x00;                  // |[gSLow
  delay_us(100);                       // 100us wait
}
void write_Data(byte cs,byte ch)
{
  LCD_CTRL_PORT = (cs & CS_ALL) | RS;  // CS{RSo
  SetE(1);                             // E High
  LCD_OUTPUT_PORT = ch;                // `lo
  delay_us(10);                        // 10us wait
  SetE(0);                             // E Low
  delay_us(10);                        // 10us wait
  LCD_CTRL_PORT=0x00;                  // |[gSLow
  delay_us(100);                       // 100us wait
}
void lcd_Clear(void)
{
  byte i,j;
  byte x,y;
  LCD_CTRL_PORT=0x00;                  // |[gSLow
  write_Cmd(CS_ALL,0x3f);              // R}h3FiʃIj
  write_Cmd(CS_ALL,0xc0);              // R}hC0ifBXvCJnsݒj
  x=0xB8;
  y=0x40;
  for(i=0;i<=7;i++)
  {
    write_Cmd(CS_ALL,x);               // XAhXݒ
    write_Cmd(CS_ALL,y);               // YAhXݒ
    for(j=0;j<64;j++)
    {
      write_Data(CS_ALL,0x00);         // f[^NA
    }
    x++;
  }
}

void lcd_Init(void)
{
  write_Cmd(CS_ALL,DISPLAY_ON);        // ʃI
  write_Cmd(CS_ALL,0xc0);              // fBXvCJnsݒ
  write_Cmd(CS_ALL,0xb8);              // XAhXݒ
  write_Cmd(CS_ALL,0x40);              // YAhXݒ
}
void DrawPixel(int x, int y)
{
  int point=0x01, value;
  int x1, y1, y2, y3;
  x1=x%128;
  y1=y%64;
  y2=y1/8;
  y3=y1%8;
  compare[y2][x1]|=(point<<y3);        // ɃsNZZbg
  value=compare[y2][x1];               // o̓f[^쐬
  if(x1<64)                            // CS1?
  {
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_X+y2);    // XAhXݒ
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_Y+x1);    // YAhXݒ
    write_Data(_CS1, value);           // CS1f[^o
  }
  else
  {
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_X+y2);    // XAhXݒ
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_Y+x1-64); // YAhXݒ
    write_Data(_CS2, value);           // CS2f[^o
  }
}
void ClearPixel(int x, int y)
{
  int point=0x01, value;
  int x1, y1, y2, y3;

  x1=x%128;
  y1=y%64;
  y2=y1/8;
  y3=y1%8;
  compare[y2][x1]&=(~(point<<y3));     // ɃsNZNA
  value=compare[y2][x1];               // o̓f[^쐬
  if(x1<64)                            // CS1?
  {
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_X+y2);    // XAhXݒ
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_Y+x1);    // YAhXݒ
    write_Data(_CS1, value);           // CS1f[^o
  }
  else
  {
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_X+y2);    // XAhXݒ
    write_Cmd(CS_ALL, nGraphic_Global_Cursor_Y+x1-64); // YAhXݒ
    write_Data(_CS2, value);           // CS2f[^o
  }
}
void DrawLine(int X0, int Y0, int X1, int Y1)  // `
{
  int x0=X0, y0=Y0, x1=X1, y1=Y1;
  int dy = y1 - y0;
  int dx = x1 - x0;
  int stepx, stepy;

  if (dy < 0)
  {
    dy = -dy;
    stepy = -1;
  }
  else
  {
    stepy = 1;
  }

  if(dx < 0)
  {
    dx = -dx;
    stepx = -1;
  }
  else
  {
    stepx = 1;
  }
  dy <<= 1;
  dx <<= 1;
  DrawPixel(x0, y0);

  if (dx > dy)
  {
    fraction = dy - (dx >> 1);
    while (x0 != x1)
    {
      if (fraction >= 0)
      {
        y0 += stepy;
        fraction -= dx;
      }
      x0 += stepx;
      fraction += dy;
      DrawPixel(x0, y0);
    }
  }
  else
  {
    int fraction = dx - (dy >> 1);
    while (y0 != y1)
    {
      if (fraction >= 0)
      {
        x0 += stepx;
        fraction -= dy;
      }
      y0 += stepy;
      fraction += dx;
      DrawPixel(x0, y0);
    }
  }
}
void ClearLine(int X0, int Y0, int X1, int Y1)  // NA`
{
  int x0=X0, y0=Y0, x1=X1, y1=Y1;
  int dy = y1 - y0;
  int dx = x1 - x0;
  int stepx, stepy;

  if (dy < 0)
  {
    dy = -dy;
    stepy = -1;
  }
  else
  {
    stepy = 1;
  }

  if (dx < 0)
  {
    dx = -dx;
    stepx = -1;
  }
  else
  {
    stepx = 1;
  }
  dy <<= 1;
  dx <<= 1;
  ClearPixel(x0, y0);
  if (dx > dy)
  {
    fraction = dy - (dx >> 1);
    while (x0 != x1)
    {
      if (fraction >= 0)
      {
        y0 += stepy;
        fraction -= dx;
      }
      x0 += stepx;
      fraction += dy;
      ClearPixel(x0, y0);
    }
  }
  else
  {
    fraction = dx - (dy >> 1);
    while (y0 != y1)
    {
      if (fraction >= 0)
      {
        x0 += stepx;
        fraction -= dy;
      }
      y0 += stepy;
      fraction += dx;
      ClearPixel(x0, y0);
    }
  }
}
// }``
void DrawRect(int left,int top,int right,int bottom,int bFilling)
{
  int i;
  if(bFilling)
  {
    if(left<right)
    {
      for(i=left;i<=right;i++)
      {
        DrawLine(i,top,i,bottom);
      }
    }
    else
    {
      for(i=left;i>=right;i--)
      {
        DrawLine(i,top,i,bottom);
      }
    }
  }
  else
  {
    DrawLine(left,top,left,bottom);
    DrawLine(left,top,right,top);
    DrawLine(right,bottom,right,top);
    DrawLine(right,bottom,left,bottom);
  }
}
// NA}``
void ClearRect(int left, int top, int right, int bottom)
{
  int i;
  if(left<right)
  {
    for(i=left;i<=right;i++)
    {
      ClearLine(i,top,i,bottom);
    }
  }
  else
  {
    for(i=left;i>=right;i--)
    {
      ClearLine(i,top,i,bottom);
    }
  }
}
// ~`
void DrawCircle(int x,int y,int radius,int bFilling)
{
  int a, r, cond;
  a = 0;
  r = radius;
  cond = 1 - r;
  do
  {
    if(bFilling)
    {
      DrawLine(x-a, y+r, x+a, y+r);
      DrawLine(x-a, y-r, x+a, y-r);
      DrawLine(x-r, y+a, x+r, y+a);
      DrawLine(x-r, y-a, x+r, y-a);
    }
    else
    {
      DrawPixel(a+x, r+y);
      DrawPixel(r+x, a+y);
      DrawPixel(x-a, r+y);
      DrawPixel(x-r, a+y);
      DrawPixel(x+r, y-a);
      DrawPixel(x+a, y-r);
      DrawPixel(x-a, y-r);
      DrawPixel(x-r, y-a);
    }
    if(cond < 0)
    {
      cond += 3 + 2*a++;
    }
    else
    {
      cond+= 5 + 2*(a++ - r--);
    }
  }while(a <= r);
}
// NA~`
void ClearCircle(int x, int y, int radius)
{
  int a, r, cond;
  a = 0;
  r = radius;
  cond = 1 - r;
  do
  {
    ClearLine(x-a, y+r, x+a, y+r);
    ClearLine(x-a, y-r, x+a, y-r);
    ClearLine(x-r, y+a, x+r, y+a);
    ClearLine(x-r, y-a, x+r, y-a);
    if(cond < 0)
    {
      cond += 3 + 2*a++;
    }
    else
    {
      cond+= 5 + 2*(a++ - r--);
    }
  }while(a <= r);
}